/***************************************************************************
 *   Copyright (C) 2005 by Robot Group Leipzig                             *
 *    martius@informatik.uni-leipzig.de                                    *
 *    fhesse@informatik.uni-leipzig.de                                     *
 *    der@informatik.uni-leipzig.de                                        *
 *    guettler@informatik.uni-leipzig.de                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
 *                                                                         *
 *  DESCRIPTION                                                            *
 *  Interface for any type of measure.                                     *
 *                                                                         *
 *  $Log: imeasure.h,v $
 *  Revision 1.1  2009/03/27 06:16:57  guettler
 *  support for gcc 4.3 compatibility (has to be checked), StatisticTools moves from utils to statistictools
 *										   *
 *                                                                         *
 **************************************************************************/
/*
 * imeasure.h
 *
 *  Created on: 26.01.2009
 *      Author: guettler
 */

#ifndef IMEASURE_H_
#define IMEASURE_H_

/**
 * Class used by StatisticTools.
 * Provides an interface for any kind of time series analysis.
 * Every step the StatisticTools calls step.
 * @sa StatisticTools
 * @sa HUDStatisticsManager
 * @sa AbstractMeasure
 */
class IMeasure {
public:

	virtual ~IMeasure() {};

	virtual void step() = 0;

	virtual std::string getName() const = 0;

	virtual double getValue() const = 0;

	virtual double& getValueAddress()  = 0;

	virtual void setStepSize(int newStepSize) = 0;

	virtual int getStepSize() const = 0;

	virtual long getActualStep() const = 0;
};

#endif /* IMEASURE_H_ */
