/***************************************************************************
 *   Copyright (C) 2005-2009 by Robot Group Leipzig                        *
 *    martius@informatik.uni-leipzig.de                                    *
 *    fhesse@informatik.uni-leipzig.de                                     *
 *    der@informatik.uni-leipzig.de                                        *
 *    guettler@informatik.uni-leipzig.de                                   *
 *    jhoffmann@informatik.uni-leipzig.de                                  *
 *    joergweide84@aol.com (robot12)                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
 *                                                                         *
 *   This class implements the getFitness function from IFitnessStrategy.  *
 *   The function self calculate the euclidic distance to zero of all gens *
 *   which are giving for the individual (accept only double gens!).	   *
 *                                                                         *
 *   $Log: EuclidicDistanceFitnessStrategy.h,v $
 *   Revision 1.2  2009/07/21 08:37:59  robot12
 *   add some comments
 *
 *   Revision 1.1  2009/06/15 13:58:36  robot12
 *   3 new fitness strategys and IFitnessStrategy and SumFitnessStragegy with comments.
 *
 *
 ***************************************************************************/

#ifndef EUCLIDICDISTANCEFITNESSSTRATEGY_H_
#define EUCLIDICDISTANCEFITNESSSTRATEGY_H_

//forward declaration
class Individual;

//ga_tools include
#include "IFitnessStrategy.h"

/**
 * This fitness strategy calculate from all double gens (IValue<double>) the euclidic
 * distance to zero.
 */
class EuclidicDistanceFitnessStrategy: public IFitnessStrategy {
public:
	/**
	 * default constructor
	 * do nothing
	 */
	EuclidicDistanceFitnessStrategy();

	/**
	 * default destructor
	 * do nothing
	 */
	virtual ~EuclidicDistanceFitnessStrategy();

	/**
	 * this function calculate the euclidic distance of all double gen which
	 * individual has.
	 * @param individual (const Individual*) the Individual
	 * @return (double) the distance
	 */
	virtual double getFitness(const Individual* individual);
};

#endif /* EUCLIDICDISTANCEFITNESSSTRATEGY_H_ */
