package unileipzig.isys.neuronviz.kernel;

import java.util.EventObject;

/**
 * Class DataEvent
 * 
 */
public class DataEvent extends EventObject
{
	private String dataLine;

	public DataEvent(Object source, String dataLine)
	{
		super(source);
		this.dataLine = dataLine;
	}

	public String getDataLine()
	{
		return dataLine;
	}
}
