package unileipzig.isys.neuronviz.kernel;

public interface ParseListener
{
	public void createNeuralNet(ParseEvent e);
	public void createLayer(ParseEvent e);
	public void createNeuron(ParseEvent e);
	public void createConnection(ParseEvent e);
	public void descriptionTerminated(ParseEvent e);
	public void unknownCommand(ParseEvent e);
	public void channelReceived(ParseEvent e);
	public void quitReceived(ParseEvent e);
}

