#ifndef TEST_HARNESS_H
#define TEST_HARNESS_H

#include "Test.h"
#include "TypedTestLauncher.h"
#include "TestResults.h"

#include "TestMacros.h"
#include "CheckMacros.h"

#include "TestRunner.h"

#endif

