package unileipzig.isys.neuronviz.kernel;

import java.util.EventObject;

public class ParseEvent extends EventObject
{
	private String [] parameterValues = null;
	private String parseMessage = null;
	private String [] channelDescription = null;
	private int parseType = 0;
	static public final int NEURALNET_TYPE = 1;
	static public final int NEURON_TYPE = 2;
	static public final int LAYER_TYPE = 3;
	static public final int CONNECTION_TYPE = 4;
	static public final int TERMINATED_TYPE = 5;
	static public final int UNKNOWN_TYPE = 6;
	static public final int RECEIVED_CHANNEL_TYPE = 7;
	static public final int QUIT_TYPE = 8;	
	
	public ParseEvent(Object source, String [] parameters, int type)
	{
		super(source);
		this.parameterValues = parameters;
		this.parseType = type;
	}

	public String [] getParameterValues()
	{
		return parameterValues;
	}

	public int getParseType()
	{
		return parseType;
	}

	public void setParseType(int parseType)
	{
		this.parseType = parseType;
	}

	public String getParseMessage()
	{
		return parseMessage;
	}

	public void setParseMessage(String parseMessage)
	{
		this.parseMessage = parseMessage;
	}

	public String [] getChannelDescription()
	{
		return channelDescription;
	}

	public void setChannelDescription(String [] channelDescription)
	{
		this.channelDescription = channelDescription;
	}
}
