/***************************************************************************
 *   Copyright (C) 2005 by Robot Group Leipzig                             *
 *    martius@informatik.uni-leipzig.de                                    *
 *    fhesse@informatik.uni-leipzig.de                                     *
 *    der@informatik.uni-leipzig.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
 *                                                                         *
 *  DESCRIPTION                                                            *
 *                                                                         *
 *   $Log: invertablemodel.h,v $
 *   Revision 1.6  2008/05/02 17:20:04  martius
 *   *** empty log message ***
 *
 *   Revision 1.5  2007/04/02 15:24:14  der
 *   inversion
 *
 *   Revision 1.4  2007/02/20 15:41:06  martius
 *   big model stuff, elman and co
 *
 *   Revision 1.3  2006/08/04 15:16:13  martius
 *   documentation
 *
 *   Revision 1.2  2006/07/20 17:14:34  martius
 *   removed std namespace from matrix.h
 *   storable interface
 *   abstract model and invertablemodel as superclasses for networks
 *
 *   Revision 1.1  2006/07/18 15:45:40  martius
 *   invertable models provide the linear response function (jacobian)
 *
 *                                                                 *
 ***************************************************************************/
#ifndef __INVERTABLENN_H
#define __INVERTABLENN_H

#include "matrix.h"
#include "abstractmodel.h"

/** abstract class (interface) for invertable models. 
    Invertable models provide a linear response function (jacobian)
*/
class InvertableModel : public AbstractModel {
 public: 
  InvertableModel() {};
  InvertableModel(const std::string& name, const std::string& revision)
    : AbstractModel(name, revision) {}
  virtual ~InvertableModel(){};

  /** calculates the partial derivative of the of the output with repect to the input (Jacobi matrix).

      \f[J_{ij} = \frac{\partial output_i}{\partial input_j}\f]

      The result is a matrix of dimension (outputdim x inputdim)
   */
  virtual const matrix::Matrix response(const matrix::Matrix& input) const = 0;

  /** calculates the input shift v to given output shift xsi via pseudo inversion.

      \f[o+\xi = \pi(i+v)\f]

      The result is a vector of dimension inputdim
   */
  virtual const matrix::Matrix inversion(const matrix::Matrix& input, const matrix::Matrix& xsi) const = 0;


};


#endif
