package unileipzig.isys.neuronviz.kernel;

import unileipzig.isys.neuronviz.kernel.DataEvent;

/**
 * Interface DataListener
 * 
 */
public interface DataListener
{
	/**
	 * 
	 * @param e
	 * @return
	 */
	public void dataReceived(DataEvent e);
	/**
	 * @see DataStreamer modes
	 * @param mode eiter FILE_MODE or PIPE_MODE
	 * @return
	 */
	public void streamingMode(int mode);

}
