//Microsoft Developer Studio generated resource script.
//
#include "resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
//#include "afxres.h"

// added by RLS to make this work with windres
#include "winresrc.h"
#define IDC_STATIC (-1)

/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// English (U.S.) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
#ifdef _WIN32
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)
#endif //_WIN32

/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_ABOUT DIALOG DISCARDABLE  0, 0, 257, 105
STYLE DS_MODALFRAME | WS_POPUP | WS_VISIBLE | WS_CAPTION | WS_SYSMENU
CAPTION "About"
FONT 8, "MS Sans Serif"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,200,84,50,14
    LTEXT           "Simulation test environment",IDC_STATIC,7,7,243,8
    LTEXT           "Change the camera position by clicking + dragging in the main window.",
                    IDC_STATIC,7,24,243,8
    LTEXT           "Left button - pan and tilt.",IDC_STATIC,25,37,225,8
    LTEXT           "Right button - forward and sideways.",IDC_STATIC,25,48,
                    225,8
    LTEXT           "Left + Right button (or middle button) - sideways and up.",
                    IDC_STATIC,25,59,225,8
END


#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE DISCARDABLE
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE DISCARDABLE
BEGIN
    //"#include ""afxres.h""\r\n"
    "\0"
END

3 TEXTINCLUDE DISCARDABLE
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Menu
//

IDR_MENU1 MENU DISCARDABLE
BEGIN
    POPUP "&File"
    BEGIN
        MENUITEM "&Exit\tCtrl+X",               IDM_EXIT
    END
    POPUP "&Simulation"
    BEGIN
        MENUITEM "&Pause\tCtrl+P",              IDM_PAUSE
        MENUITEM "Single Step\tCtrl+O",         IDM_SINGLE_STEP
        MENUITEM "Performance &Monitor",        IDM_PERF_MONITOR, GRAYED
        MENUITEM SEPARATOR
        MENUITEM "&Shadows\tCtrl+S",            IDM_SHADOWS, CHECKED
        MENUITEM "&Textures\tCtrl+T",           IDM_TEXTURES, CHECKED
        MENUITEM SEPARATOR
        MENUITEM "S&ave Settings",              IDM_SAVE_SETTINGS, GRAYED
    END
    POPUP "&Help"
    BEGIN
        MENUITEM "&About",                      IDM_ABOUT
    END
END


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO DISCARDABLE
BEGIN
    IDD_ABOUT, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 250
        VERTGUIDE, 25
        TOPMARGIN, 7
        BOTTOMMARGIN, 98
    END
END
#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Accelerator
//

IDR_ACCELERATOR1 ACCELERATORS DISCARDABLE
BEGIN
    "O",            IDM_SINGLE_STEP,        VIRTKEY, CONTROL, NOINVERT
    "P",            IDM_PAUSE,              VIRTKEY, CONTROL, NOINVERT
    "S",            IDM_SHADOWS,            VIRTKEY, CONTROL, NOINVERT
    "T",            IDM_TEXTURES,           VIRTKEY, CONTROL, NOINVERT
    "X",            IDM_EXIT,               VIRTKEY, CONTROL, NOINVERT
END

#endif    // English (U.S.) resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

